@@ -1,3 +1,4 @@
+#[cfg(feature = "generate")]
 extern crate bindgen;
 extern crate cc;
 extern crate pkg_config;
@@ -83,14 +84,15 @@ fn compile() {
         );
     }
 
-    let os = env::var("CARGO_CFG_TARGET_OS").unwrap();
+    let arch = env::var("CARGO_CFG_TARGET_ARCH");
+    let os = env::var("CARGO_CFG_TARGET_OS");
     let bits = env::var("CARGO_CFG_TARGET_POINTER_WIDTH").unwrap();
-    if os == "windows" {
+    if let Ok("windows") = os.as_ref().map(String::as_str) {
         fs::copy(src.join(format!("config.h.win{}", bits)), config_h)
             .expect("Can't copy config.h.win??");
     } else {
-        let family = env::var("CARGO_CFG_TARGET_FAMILY").unwrap();
-        if family == "unix" {
+        let family = env::var("CARGO_CFG_TARGET_FAMILY");
+        if let Ok("unix") = family.as_ref().map(String::as_str) {
             cc.define("HAVE_UNISTD_H", Some("1"));
             cc.define("HAVE_SYS_TYPES_H", Some("1"));
             cc.define("HAVE_SYS_TIME_H", Some("1"));
@@ -118,6 +120,15 @@ fn compile() {
         )
         .expect("Can't write config.h to OUT_DIR");
     }
+    if let Ok("wasm32") = arch.as_ref().map(String::as_str) {
+        cc.define("ONIG_DISABLE_DIRECT_THREADING", Some("1"));
+        if let Ok("unknown") = os.as_ref().map(String::as_str) {
+            cc.define(
+                "ONIG_EXTERN",
+                Some(r#"__attribute__((visibility("default")))"#),
+            );
+        }
+    }
 
     cc.include(out_dir); // Read config.h from there
     cc.include(src);
@@ -134,7 +145,6 @@ fn compile() {
         "regtrav.c",
         "regversion.c",
         "st.c",
-        "regposerr.c",
         "onig_init.c",
         "unicode.c",
         "ascii.c",
@@ -179,18 +189,27 @@ fn compile() {
 
     if cfg!(feature = "posix-api") {
         cc.file(src.join("regposix.c"));
+        cc.file(src.join("regposerr.c"));
     }
 
     cc.warnings(false); // not actionable by the end user
     cc.compile("onig");
 }
 
+#[cfg(not(feature = "generate"))]
+fn bindgen_headers(_path: &str) {}
+
+#[cfg(feature = "generate")]
 fn bindgen_headers(path: &str) {
-    let bindings = bindgen::Builder::default()
+    let arch = env::var("CARGO_CFG_TARGET_ARCH");
+    let mut bindgen = bindgen::Builder::default()
         .header(path)
         .derive_eq(true)
-        .generate()
-        .expect("bindgen");
+        .layout_tests(false);
+    if let Ok("wasm32") = arch.as_ref().map(String::as_str) {
+        bindgen = bindgen.clang_arg("-fvisibility=default");
+    }
+    let bindings = bindgen.generate().expect("bindgen");
     let out_dir = env::var_os("OUT_DIR").expect("OUT_DIR");
     let out_path = Path::new(&out_dir);
     bindings
@@ -204,21 +223,32 @@ pub fn main() {
 
     if require_pkg_config || link_type == Some(LinkType::Dynamic) {
         let mut conf = Config::new();
-        conf.atleast_version("6.8.0");
+        // dynamically-generated headers can work with an older version
+        // pre-generated headers are for the latest
+        conf.atleast_version(if cfg!(feature = "generate") {
+            "6.8.0"
+        } else {
+            "6.9.3"
+        });
         if link_type == Some(LinkType::Static) {
             conf.statik(true);
         }
         match conf.probe("oniguruma") {
             Ok(lib) => {
-                for path in lib.include_paths {
+                for path in &lib.include_paths {
                     let header = path.join("oniguruma.h");
                     if header.exists() {
                         bindgen_headers(&header.display().to_string());
-                        break;
+                        return;
                     }
                 }
-                return
-            },
+                if require_pkg_config {
+                    panic!(
+                        "Unable to find oniguruma.h in include paths from pkg-config: {:?}",
+                        lib.include_paths
+                    );
+                }
+            }
             Err(ref err) if require_pkg_config => {
                 panic!("Unable to find oniguruma in pkg-config, and RUSTONIG_SYSTEM_LIBONIG is set: {}", err);
             }
